--
-- Options.lua
--
-- Handles all the options for GalvinUnitBars.
-- Handles all the options for GalvinUnitBars.

-------------------------------------------------------------------------------
-- GUB   shared data table between all parts of the addon
-------------------------------------------------------------------------------
local MyAddon, GUB = ...

GUB.Options = {}

local UnitBarsF = GUB.UnitBars.UnitBarsF
local LSM = GUB.UnitBars.LSM
local Defaults = GUB.UnitBars.Defaults

-------------------------------------------------------------------------------
-- Locals
--
-- UnitBarsOptionsFrame          Main options frame used by this addon.
-- UnitBarsProfilesFrame         Used to show the profile settings in the blizzard
--                               options tree.
-- UnitBarsOptions               Table containing the options data to display in
--                               options frame.
-- UnitBarsSlashOptions          Options only used by slash commands. This is accessed
--                               by typing '/gub'.
-- UnitBarsProfileOptions        Options for the profiles.
--
-- FontStyleDropdown             Table used for the dialog drop down for FontStyles.
-- FontHAlignDropDown            Table used for the dialog drop down for font horizontal alignment.
-- TextTypeDropdown              Table used for the dialog drop down for Healh and power text type.
--
-- FontOffsetXMax
-- FontOffsetYMax
-- FontOffsetXMin
-- FontOffsetYMin
-- FontShadowOffsetMin
-- FontShadowOffsetMax
--                               Minimum and maximum sizes that can be entered thru the options panel
--                               for font offset.
-------------------------------------------------------------------------------

-- Constants
local AddonName = GetAddOnMetadata(MyAddon, "Title")
local AddonVersion = GetAddOnMetadata(MyAddon, "Version")

local AddonOptionsName = MyAddon .. 'options'
local AddonProfileName = MyAddon .. 'profile'
local AddonSlashName = MyAddon .. 'slash'

local FontOffsetXMin = -100
local FontOffsetXMax = 100
local FontOffsetYMin = -50
local FontOffsetYMax = 50

local FontShadowOffsetMin = 0
local FontShadowOffsetMax = 10

local UnitBarWidthMin = 30
local UnitBarWidthMax = 500
local UnitBarHeightMin = 25
local UnitBarHeightMax = 100

local UnitBarPaddingMin = -20
local UnitBarPaddingMax = 20

local UnitBarBorderSizeMin = 2
local UnitBarBorderSizeMax = 32

local UnitBarFontSizeMin = 6
local UnitBarFontSizeMax = 24

--local UnitBarsOptionsFrame = nil
local UnitBarsProfilesFrame = nil

local UnitBarsProfileOptions = nil

local UnitBars = nil

local FontStyleDropdown = {
  NONE = "None",
  OUTLINE = "Outline",
  THICKOUTLINE = "Thick Outline",
  ["NONE, MONOCHROME"] = "No Outline, Mono",
  ["OUTLINE, MONOCHROME"] = "Outline, Mono",
  ["THICKOUTLINE, MONOCHROME"] = "Thick Outline, Mono",
}
local FontHAlignDropDown = {
  LEFT = 'Left',
  CENTER = 'Center',
  RIGHT = 'Right'
}
local TextTypeDropdown = {
  none = 'No value',
  percent = 'Percentage',
  max = 'Value / Max Value',
  whole = 'Whole Number'
}

--*****************************************************************************
--
-- Options Setter/Getter functions
--
--*****************************************************************************


--*****************************************************************************
--
-- Ace3 options tables
--
--*****************************************************************************

-------------------------------------------------------------------------------
-- CreateUnitBarsSlashOptions()
--
-- Returns a slash options table for unitbars.
-------------------------------------------------------------------------------

--=============================================================================
-- UnitBar slashcommand options group
--=============================================================================
local function CreateUnitBarsSlashOptions()
  local SlashOptions = {
    type = 'group',
    name = 'slash command',
    order = 1,
    args = {
      about = {
        type = 'execute',
        name = 'about',
        order = 2,
        func = function()
                print(AddonName, 'Version ', AddonVersion)
               end,
      },
      config = {
        type = 'execute',
        name = 'config',
        order = 2,
        desc = 'Opens a movable options frame',
        func = function()
                 if UnitBarsOptionsFrame:IsVisible() then
                   GUB:Print('Must close blizzard options first')
                 else
                   -- Open a movable options frame.
                   LibStub('AceConfigDialog-3.0'):Open(AddonOptionsName)
                 end
               end,

      },
    },
  }
  return SlashOptions
end

-------------------------------------------------------------------------------
-- CreateUnitBarsOptions()
--
-- Returns an GUI options table for unitbars.
-------------------------------------------------------------------------------
local function CreateUnitBarsOptions()

  UnitBarsProfileOptions.order = 100

--=============================================================================
--=============================================================================
--Galvin's UnitBars Group.
--=============================================================================
--=============================================================================
  local GUIOptions = {
    name = AddonName,
    type = 'group',
    order = 1,
    childGroups = 'tab',
    args = {
--=============================================================================
-------------------------------------------------------------------------------
--    GENERAL group.
-------------------------------------------------------------------------------
--=============================================================================
      General = {
        name = 'General',
        type = 'group',
        order = 1,
        get = function(Info)
                return UnitBars[Info[#Info]]
              end,
        set = function(Info, Value)
                UnitBars[Info[#Info]] = Value
                GUB.UnitBars:UnitBarsSetAllOptions()
                GUB:UnitBarsUpdateStatus()
              end,
        args = {
          IsLocked = {
            type = 'toggle',
            name = 'Lock Bars',
            order = 1,
            desc = 'Prevent bars from being dragged around',
          },
          IsClamped = {
            type = 'toggle',
            name = 'Screen Clamp',
            order = 2,
            desc = 'Prevent bars from going off the screen',
          },
          IsGrouped = {
            type = 'toggle',
            name = 'Group Drag',
            order = 3,
            desc = 'Drag all the bars as one instead of one at a time',
          },
          HideTooltips = {
            type = 'toggle',
            name = 'Hide Tooltips',
            order = 4,
            desc = 'Turns off mouse over tooltips when bars are not locked',
          },
          SmoothUpdate = {
            type = 'toggle',
            name = 'Smooth Update',
            order = 5,
            desc = 'Health and power bars will update smoothly if checked',
          },
          FadeOutTime = {
            type = 'range',
            name = 'Fadeout Time',
            order = 6,
            desc = 'The amount of time in seconds to fade out a bar',
            min = 0,
            max = 5,
            step = 1,
            get = function()
                    return UnitBars.FadeOutTime
                  end,
            set = function(Info, Value)
                    UnitBars.FadeOutTime = Value
                    GUB.UnitBars:UnitBarsSetAllOptions()
                  end,
          },
        },
      },
--=============================================================================
-------------------------------------------------------------------------------
--    BARS group.
-------------------------------------------------------------------------------
--=============================================================================
      UnitBars = {
        type = 'group',
        name = 'Bars',
        order = 2,
        args = {
--=============================================================================
--        Player Health group.
--=============================================================================
          PlayerHealth = {
            type = 'group',
            name = 'Player Health',
            order = 1,
            args = {
-------------------------------------------------------------------------------
--            Status - Player Health Group.
-------------------------------------------------------------------------------
              Status = {
                type = 'group',
                name = 'Status',
                dialogInline = true,
                order = 1,
                get = function(Info)
                        return UnitBars.PlayerHealth.Status[Info[#Info]]
                      end,
                set = function(Info, Value)
                        UnitBars.PlayerHealth.Status[Info[#Info]] = Value

                        -- Must do a status check/update.
                        GUB:UnitBarsUpdateStatus()
                      end,
                args = {
                  ShowNever = {
                    type = 'toggle',
                    name = 'Never Show',
                    order = 1,
                    desc = 'Disables and hides the bar',
                  },
                  HideWhenDead = {
                    type = 'toggle',
                    name = "Hide when Dead",
                    order = 2,
                    desc = "Hides the bar when you're dead",
                  },
                  HideInVehicle = {
                    type = 'toggle',
                    name = 'Hide when in a Vehicle',
                    order = 3,
                    desc = "Hides the bar when your're in a vehicle",
                  },
                  ShowAlways = {
                    type = 'toggle',
                    name = 'Show Always',
                    order = 4,
                    desc = 'Bar will always be shown',
                  },
                  ShowActive = {
                    type = 'toggle',
                    name = 'Show when Active',
                    order = 5,
                    desc = 'Bar will be shown if there is activty in it',
                  },
                  HideNoCombat = {
                    type = 'toggle',
                    name = 'Hide out of Combat',
                    order = 6,
                    desc = 'When not in combat the bar will be hidden',
                  },
                },
              },
              TextType = {
                type = 'select',
                name = 'Text Type',
                order = 2,
                values = TextTypeDropdown,
                style = 'dropdown',
                desc = 'Changes the look of the value text in the bar',
                get = function()
                        return UnitBars.PlayerHealth.TextType
                      end,
                set = function(Info, Value)
                        UnitBars.PlayerHealth.TextType = Value
                           -- Redraw the bar to show the texttype change.
                        UnitBarsF.PlayerHealth:Update()
                      end,

              },
              BlankLine = {
                type = 'header',
                name = '',
                order = 3,
              },
              Reset = {
                type = 'execute',
                name = 'Reset to Defaults',
                order = 4,
                desc = 'Resets back to the defaults for this bar without changing its location',
                confirm = true,
                func = function()
                         -- Preserve bar location
                         local UB = UnitBars.PlayerHealth
                         local x, y =  UB.x, UB.y

                         GUB.UnitBars:CopyTableValues(Defaults.profile.PlayerHealth, UB)

                         UB.x, UB.y = x, y

                         -- Redo everything to show any possible changes.
                         GUB:OnEnable()
                       end,
              },
-------------------------------------------------------------------------------
--            Background - Player Health Group.
-------------------------------------------------------------------------------
              Background = {
                type = 'group',
                name = 'Background',
                order = 10,
                args = {
                  Textures = {
                    type = 'group',
                    name = 'Textures',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.PlayerHealth.BackdropSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerHealth.BackdropSettings[Info[#Info]] = Value
                            UnitBarsF.PlayerHealth:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      BdTexture = {
                        type = 'select',
                        name = 'Border',
                        order = 1,
                        dialogControl = 'LSM30_Border',
                        values = LSM:HashTable('border'),
                      },
                      BgTexture = {
                        type = 'select',
                        name = 'Background',
                        order = 2,
                        dialogControl = 'LSM30_Background',
                        values = LSM:HashTable('background'),
                      },
                      BdSize = {
                        type = 'range',
                        name = 'Border Thickness',
                        order = 3,
                        min = UnitBarBorderSizeMin,
                        max = UnitBarBorderSizeMax,
                        step = 2,
                      },
                      BgColor = {
                        type = 'color',
                        name = 'Background Color',
                        order = 4,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.PlayerHealth.BgColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.PlayerHealth.BgColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.PlayerHealth:SetAttr('bg', 'color')
                              end,
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 2,
                    get = function(Info)
                            return UnitBars.PlayerHealth.BackdropSettings.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerHealth.BackdropSettings.Padding[Info[#Info]] = Value
                            UnitBarsF.PlayerHealth:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      Left = {
                      type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Text - Player Health Group.
-------------------------------------------------------------------------------
              Text = {
                type = 'group',
                name = 'Text',
                order = 11,
                args = {
                  Font = {
                    type = 'group',
                    name = 'Font',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.PlayerHealth.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerHealth.FontSettings[Info[#Info]] = Value
                            UnitBarsF.PlayerHealth:SetAttr('text', 'font')
                          end,
                    args = {
                      FontType = {
                        type = 'select',
                        name = 'Type',
                        order = 1,
                        dialogControl = 'LSM30_Font',
                        values = LSM:HashTable('font'),
                      },
                      FontStyle = {
                        type = 'select',
                        name = 'Style',
                        order = 2,
                        style = 'dropdown',
                        values = FontStyleDropdown,
                      },
                      FontHAlign = {
                        type = 'select',
                        name = 'Horizontal Alignment',
                        order = 3,
                        style = 'dropdown',
                        values = FontHAlignDropDown,
                      },
                      FontSize = {
                        type = 'range',
                        name = 'Size',
                        order = 4,
                        min = UnitBarFontSizeMin,
                        max = UnitBarFontSizeMax,
                        step = 1,
                      },
                      TextColor = {
                        type = 'color',
                        name = 'Color',
                        order = 5,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.PlayerHealth.TextColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.PlayerHealth.TextColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.PlayerHealth:SetAttr('text', 'color')
                              end,
                      },
                    },
                  },
                  Offsets = {
                    type = 'group',
                    name = 'Offsets',
                    dialogInline = true,
                    order = 2,
                    get = function(Info)
                            return UnitBars.PlayerHealth.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerHealth.FontSettings[Info[#Info]] = Value
                            UnitBarsF.PlayerHealth:SetAttr('text', 'font')
                          end,
                    args = {
                      OffsetX = {
                        type = 'range',
                        name = 'Horizonal',
                        order = 1,
                        min = FontOffsetXMin,
                        max = FontOffsetXMax,
                        step = 1,
                      },
                      OffsetY = {
                        type = 'range',
                        name = 'Vertical',
                        order = 2,
                        min = FontOffsetYMin,
                        max = FontOffsetYMax,
                        step = 1,
                      },
                      ShadowOffset = {
                        type = 'range',
                        name = 'Shadow',
                        order = 3,
                        min = FontShadowOffsetMin,
                        max = FontShadowOffsetMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Bar - Player Health Group.
-------------------------------------------------------------------------------
              Bar = {
                type = 'group',
                name = 'Bar',
                order = 12,
                args = {
                  General = {
                    type = 'group',
                    name = 'General',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.PlayerHealth[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerHealth[Info[#Info]] = Value
                            UnitBarsF.PlayerHealth:SetAttr('bar', 'size')
                          end,
                    args = {
                      StatusBarTexture = {
                        type = 'select',
                        name = 'Bar Texture',
                        order = 1,
                        dialogControl = 'LSM30_Statusbar',
                        values = LSM:HashTable('statusbar'),
                        get = function()
                                return UnitBars.PlayerHealth.StatusBarTexture
                              end,
                        set = function(Info, Value)
                                UnitBars.PlayerHealth.StatusBarTexture = Value
                                UnitBarsF.PlayerHealth:SetAttr('bar', 'texture')
                              end,
                      },
                      Width = {
                        type = 'range',
                        name = 'Width',
                        order = 2,
                        min = UnitBarWidthMin,
                        max = UnitBarWidthMax,
                        step = 1,
                      },
                      Height = {
                        type = 'range',
                        name = 'Height',
                        order = 3,
                        min = UnitBarHeightMin,
                        max = UnitBarHeightMax,
                        step = 1,
                      },
                      BarColor = {
                        type = 'color',
                        name = 'Color',
                        order = 4,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.PlayerHealth.BarColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.PlayerHealth.BarColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.PlayerHealth:SetAttr('bar', 'color')
                              end,
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 2,
                    get = function(Info)
                            return UnitBars.PlayerHealth.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerHealth.Padding[Info[#Info]] = Value
                            UnitBarsF.PlayerHealth:SetAttr('bar', 'padding')
                          end,
                    args = {
                      Left = {
                        type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
            },
          },
--=============================================================================
--        Player Power group.
--=============================================================================
          PlayerPower = {
            type = 'group',
            name = 'Player Power',
            order = 2,
            args = {
-------------------------------------------------------------------------------
--            Status - Player Power Group.
-------------------------------------------------------------------------------
              Status = {
                type = 'group',
                name = 'Status',
                dialogInline = true,
                order = 1,
                get = function(Info)
                        return UnitBars.PlayerPower.Status[Info[#Info]]
                      end,
                set = function(Info, Value)
                        UnitBars.PlayerPower.Status[Info[#Info]] = Value

                        -- Must do a status check/update.
                        GUB:UnitBarsUpdateStatus()
                      end,
                args = {
                  ShowNever = {
                    type = 'toggle',
                    name = 'Never Show',
                    order = 1,
                    desc = 'Disables and hides the bar',
                  },
                  HideWhenDead = {
                    type = 'toggle',
                    name = "Hide when Dead",
                    order = 2,
                    desc = "Hides the bar when you're dead",
                  },
                  HideInVehicle = {
                    type = 'toggle',
                    name = 'Hide when in a Vehicle',
                    order = 3,
                    desc = "Hides the bar when your're in a vehicle",
                  },
                  ShowAlways = {
                    type = 'toggle',
                    name = 'Show Always',
                    order = 4,
                    desc = 'Bar will always be shown',
                  },
                  ShowActive = {
                    type = 'toggle',
                    name = 'Show when Active',
                    order = 5,
                    desc = 'Bar will be shown if there is activty in it',
                  },
                  HideNoCombat = {
                    type = 'toggle',
                    name = 'Hide out of Combat',
                    order = 6,
                    desc = 'When not in combat the bar will be hidden',
                  },
                },
              },
              TextType = {
                type = 'select',
                name = 'Text Type',
                order = 2,
                values = TextTypeDropdown,
                style = 'dropdown',
                desc = 'Changes the look of the value text in the bar',
                get = function()
                        return UnitBars.PlayerPower.TextType
                      end,
                set = function(Info, Value)
                        UnitBars.PlayerPower.TextType = Value
                           -- Redraw the bar to show the texttype change.
                        UnitBarsF.PlayerPower:Update()
                      end,

              },
              BlankLine = {
                type = 'header',
                name = '',
                order = 3,
              },
              Reset = {
                type = 'execute',
                name = 'Reset to Defaults',
                order = 4,
                desc = 'Resets back to the defaults for this bar without changing its location',
                confirm = true,
                func = function()
                         -- Preserve bar location
                         local UB = UnitBars.PlayerPower
                         local x, y =  UB.x, UB.y

                         GUB.UnitBars:CopyTableValues(Defaults.profile.PlayerPower, UB)

                         UB.x, UB.y = x, y

                         -- Redo everything to show any possible changes.
                         GUB:OnEnable()
                       end,
              },
-------------------------------------------------------------------------------
--            Background - Player Power Group.
-------------------------------------------------------------------------------
              Background = {
                type = 'group',
                name = 'Background',
                order = 10,
                args = {
                  Textures = {
                    type = 'group',
                    name = 'Textures',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.PlayerPower.BackdropSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerPower.BackdropSettings[Info[#Info]] = Value
                            UnitBarsF.PlayerPower:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      BdTexture = {
                        type = 'select',
                        name = 'Border',
                        order = 1,
                        dialogControl = 'LSM30_Border',
                        values = LSM:HashTable('border'),
                      },
                      BgTexture = {
                        type = 'select',
                        name = 'Background',
                        order = 2,
                        dialogControl = 'LSM30_Background',
                        values = LSM:HashTable('background'),
                      },
                      BdSize = {
                        type = 'range',
                        name = 'Border Thickness',
                        order = 3,
                        min = UnitBarBorderSizeMin,
                        max = UnitBarBorderSizeMax,
                        step = 2,
                      },
                      BgColor = {
                        type = 'color',
                        name = 'Background Color',
                        order = 4,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.PlayerPower.BgColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.PlayerPower.BgColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.PlayerPower:SetAttr('bg', 'color')
                              end,
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 2,
                    get = function(Info)
                            return UnitBars.PlayerPower.BackdropSettings.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerPower.BackdropSettings.Padding[Info[#Info]] = Value
                            UnitBarsF.PlayerPower:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      Left = {
                      type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Text - Player Power Group.
-------------------------------------------------------------------------------
              Text = {
                type = 'group',
                name = 'Text',
                order = 11,
                args = {
                  Font = {
                    type = 'group',
                    name = 'Font',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.PlayerPower.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerPower.FontSettings[Info[#Info]] = Value
                            UnitBarsF.PlayerPower:SetAttr('text', 'font')
                          end,
                    args = {
                      FontType = {
                        type = 'select',
                        name = 'Type',
                        order = 1,
                        dialogControl = 'LSM30_Font',
                        values = LSM:HashTable('font'),
                      },
                      FontStyle = {
                        type = 'select',
                        name = 'Style',
                        order = 2,
                        style = 'dropdown',
                        values = FontStyleDropdown,
                      },
                      FontHAlign = {
                        type = 'select',
                        name = 'Horizontal Alignment',
                        order = 3,
                        style = 'dropdown',
                        values = FontHAlignDropDown,
                      },
                      FontSize = {
                        type = 'range',
                        name = 'Size',
                        order = 4,
                        min = UnitBarFontSizeMin,
                        max = UnitBarFontSizeMax,
                        step = 1,
                      },
                      TextColor = {
                        type = 'color',
                        name = 'Color',
                        order = 5,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.PlayerPower.TextColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.PlayerPower.TextColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.PlayerPower:SetAttr('text', 'color')
                              end,
                      },
                    },
                  },
                  Offsets = {
                    type = 'group',
                    name = 'Offsets',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.PlayerPower.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerPower.FontSettings[Info[#Info]] = Value
                            UnitBarsF.PlayerPower:SetAttr('text', 'font')
                          end,
                    args = {
                      OffsetX = {
                        type = 'range',
                        name = 'Horizonal',
                        order = 4,
                        min = FontOffsetXMin,
                        max = FontOffsetXMax,
                        step = 1,
                      },
                      OffsetY = {
                        type = 'range',
                        name = 'Vertical',
                        order = 5,
                        min = FontOffsetYMin,
                        max = FontOffsetYMax,
                        step = 1,
                      },
                      ShadowOffset = {
                        type = 'range',
                        name = 'Shadow',
                        order = 6,
                        min = FontShadowOffsetMin,
                        max = FontShadowOffsetMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Bar - Player Power Group.
-------------------------------------------------------------------------------
              Bar = {
                type = 'group',
                name = 'Bar',
                order = 12,
                args = {
                  General = {
                    type = 'group',
                    name = 'General',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.PlayerPower[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerPower[Info[#Info]] = Value
                            UnitBarsF.PlayerPower:SetAttr('bar', 'size')
                          end,
                    args = {
                      StatusBarTexture = {
                        type = 'select',
                        name = 'Bar Texture',
                        order = 1,
                        dialogControl = 'LSM30_Statusbar',
                        values = LSM:HashTable('statusbar'),
                        get = function()
                                return UnitBars.PlayerPower.StatusBarTexture
                              end,
                        set = function(Info, Value)
                                UnitBars.PlayerPower.StatusBarTexture = Value
                                UnitBarsF.PlayerPower:SetAttr('bar', 'texture')
                              end,
                      },
                      Width = {
                        type = 'range',
                        name = 'Width',
                        order = 2,
                        min = UnitBarWidthMin,
                        max = UnitBarWidthMax,
                        step = 1,
                      },
                      Height = {
                        type = 'range',
                        name = 'Height',
                        order = 3,
                        min = UnitBarHeightMin,
                        max = UnitBarHeightMax,
                        step = 1,
                      },
                    },
                  },
                  PowerColors = {
                    type = 'group',
                    name = 'Power Colors',
                    order = 2,
                    dialogInline = true,
                    get = function(Info)
                            -- Info.arg[1] = power color index.
                            local c = UnitBars.PlayerPower.BarColor[Info.arg[1]]
                            return c.r, c.g, c.b, c.a
                          end,
                    set = function(Info, r, g, b, a)
                            local c = UnitBars.PlayerPower.BarColor[Info.arg[1]]
                            c.r, c.g, c.b, c.a = r, g, b, a

                            -- Update the bar to show the current power color change in real time.
                            UnitBarsF.PlayerPower:Update()
                          end,
                    args = {
                      ManaColor = {
                        type = 'color',
                        name = 'Mana',
                        order = 1,
                        hasAlpha = true,
                        arg = {0},
                      },
                      RageColor = {
                        type = 'color',
                        name = 'Rage',
                        order = 2,
                        hasAlpha = true,
                        arg = {1},
                      },
                      FocusColor = {
                        type = 'color',
                        name = 'Focus',
                        order = 3,
                        hasAlpha = true,
                        arg = {2},
                      },
                      EnergyColor = {
                        type = 'color',
                        name = 'Energy',
                        order = 4,
                        hasAlpha = true,
                        arg = {3},
                      },
                      RunicColor = {
                        type = 'color',
                        name = 'Runic Power',
                        order = 5,
                        hasAlpha = true,
                        arg = {6},
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 3,
                    get = function(Info)
                            return UnitBars.PlayerPower.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.PlayerPower.Padding[Info[#Info]] = Value
                            UnitBarsF.PlayerPower:SetAttr('bar', 'padding')
                          end,
                    args = {
                      Left = {
                        type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
            },
          },
--=============================================================================
--        Target Health group.
--=============================================================================
          TargetHealth = {
            type = 'group',
            name = 'Target Health',
            order = 3,
            args = {
-------------------------------------------------------------------------------
--            Status - Target Health group.
-------------------------------------------------------------------------------
              Status = {
                type = 'group',
                name = 'Status',
                dialogInline = true,
                order = 1,
                get = function(Info)
                        return UnitBars.TargetHealth.Status[Info[#Info]]
                      end,
                set = function(Info, Value)
                        UnitBars.TargetHealth.Status[Info[#Info]] = Value

                        -- Must do a status check/update.
                        GUB:UnitBarsUpdateStatus()
                      end,
                args = {
                  ShowNever = {
                    type = 'toggle',
                    name = 'Never Show',
                    order = 1,
                    desc = 'Disables and hides the bar',
                  },
                  HideWhenDead = {
                    type = 'toggle',
                    name = "Hide when Dead",
                    order = 2,
                    desc = "Hides the bar when you're dead",
                  },
                  HideInVehicle = {
                    type = 'toggle',
                    name = 'Hide when in a Vehicle',
                    order = 3,
                    desc = "Hides the bar when your're in a vehicle",
                  },
                  ShowAlways = {
                    type = 'toggle',
                    name = 'Show Always',
                    order = 4,
                    desc = 'Bar will always be shown',
                  },
                  ShowActive = {
                    type = 'toggle',
                    name = 'Show when Active',
                    order = 5,
                    desc = 'Bar will be shown if there is activty in it',
                  },
                  HideNoCombat = {
                    type = 'toggle',
                    name = 'Hide out of Combat',
                    order = 6,
                    desc = 'When not in combat the bar will be hidden',
                  },
                },
              },
              TextType = {
                type = 'select',
                name = 'Text Type',
                order = 2,
                values = TextTypeDropdown,
                style = 'dropdown',
                desc = 'Changes the look of the value text in the bar',
                get = function()
                        return UnitBars.TargetHealth.TextType
                      end,
                set = function(Info, Value)
                        UnitBars.TargetHealth.TextType = Value
                           -- Redraw the bar to show the texttype change.
                        UnitBarsF.TargetHealth:Update()
                      end,

              },
              BlankLine = {
                type = 'header',
                name = '',
                order = 3,
              },
              Reset = {
                type = 'execute',
                name = 'Reset to Defaults',
                order = 4,
                desc = 'Resets back to the defaults for this bar without changing its location',
                confirm = true,
                func = function()
                         -- Preserve bar location
                         local UB = UnitBars.TargetHealth
                         local x, y =  UB.x, UB.y

                         GUB.UnitBars:CopyTableValues(Defaults.profile.TargetHealth, UB)

                         UB.x, UB.y = x, y

                         -- Redo everything to show any possible changes.
                         GUB:OnEnable()
                       end,
              },
-------------------------------------------------------------------------------
--            Background - Target Health group.
-------------------------------------------------------------------------------
              Background = {
                type = 'group',
                name = 'Background',
                order = 10,
                args = {
                  Textures = {
                    type = 'group',
                    name = 'Textures',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.TargetHealth.BackdropSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetHealth.BackdropSettings[Info[#Info]] = Value
                            UnitBarsF.TargetHealth:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      BdTexture = {
                        type = 'select',
                        name = 'Border',
                        order = 1,
                        dialogControl = 'LSM30_Border',
                        values = LSM:HashTable('border'),
                      },
                      BgTexture = {
                        type = 'select',
                        name = 'Background',
                        order = 2,
                        dialogControl = 'LSM30_Background',
                        values = LSM:HashTable('background'),
                      },
                      BdSize = {
                        type = 'range',
                        name = 'Border Thickness',
                        order = 3,
                        min = UnitBarBorderSizeMin,
                        max = UnitBarBorderSizeMax,
                        step = 2,
                      },
                      BgColor = {
                        type = 'color',
                        name = 'Background Color',
                        order = 4,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.TargetHealth.BgColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.TargetHealth.BgColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.TargetHealth:SetAttr('bg', 'color')
                              end,
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 2,
                    get = function(Info)
                            return UnitBars.TargetHealth.BackdropSettings.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetHealth.BackdropSettings.Padding[Info[#Info]] = Value
                            UnitBarsF.TargetHealth:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      Left = {
                      type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Text - Target Health group.
-------------------------------------------------------------------------------
              Text = {
                type = 'group',
                name = 'Text',
                order = 11,
                args = {
                  Font = {
                    type = 'group',
                    name = 'Font',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.TargetHealth.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetHealth.FontSettings[Info[#Info]] = Value
                            UnitBarsF.TargetHealth:SetAttr('text', 'font')
                          end,
                    args = {
                      FontType = {
                        type = 'select',
                        name = 'Type',
                        order = 1,
                        dialogControl = 'LSM30_Font',
                        values = LSM:HashTable('font'),
                      },
                      FontStyle = {
                        type = 'select',
                        name = 'Style',
                        order = 2,
                        style = 'dropdown',
                        values = FontStyleDropdown,
                      },
                      FontHAlign = {
                        type = 'select',
                        name = 'Horizontal Alignment',
                        order = 3,
                        style = 'dropdown',
                        values = FontHAlignDropDown,
                      },
                      FontSize = {
                        type = 'range',
                        name = 'Size',
                        order = 4,
                        min = UnitBarFontSizeMin,
                        max = UnitBarFontSizeMax,
                        step = 1,
                      },
                      TextColor = {
                        type = 'color',
                        name = 'Color',
                        order = 5,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.TargetHealth.TextColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.TargetHealth.TextColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.TargetHealth:SetAttr('text', 'color')
                              end,
                      },
                    },
                  },
                  Offsets = {
                    type = 'group',
                    name = 'Offsets',
                    dialogInline = true,
                    order = 2,
                    get = function(Info)
                            return UnitBars.TargetHealth.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetHealth.FontSettings[Info[#Info]] = Value
                            UnitBarsF.TargetHealth:SetAttr('text', 'font')
                          end,
                    args = {
                      OffsetX = {
                        type = 'range',
                        name = 'Horizonal',
                        order = 1,
                        min = FontOffsetXMin,
                        max = FontOffsetXMax,
                        step = 1,
                      },
                      OffsetY = {
                        type = 'range',
                        name = 'Vertical',
                        order = 2,
                        min = FontOffsetYMin,
                        max = FontOffsetYMax,
                        step = 1,
                      },
                      ShadowOffset = {
                        type = 'range',
                        name = 'Shadow',
                        order = 3,
                        min = FontShadowOffsetMin,
                        max = FontShadowOffsetMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Bar - Target Health group.
-------------------------------------------------------------------------------
              Bar = {
                type = 'group',
                name = 'Bar',
                order = 12,
                args = {
                  General = {
                    type = 'group',
                    name = 'General',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.TargetHealth[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetHealth[Info[#Info]] = Value
                            UnitBarsF.TargetHealth:SetAttr('bar', 'size')
                          end,
                    args = {
                      StatusBarTexture = {
                        type = 'select',
                        name = 'Bar Texture',
                        order = 1,
                        dialogControl = 'LSM30_Statusbar',
                        values = LSM:HashTable('statusbar'),
                        get = function()
                                return UnitBars.TargetHealth.StatusBarTexture
                              end,
                        set = function(Info, Value)
                                UnitBars.TargetHealth.StatusBarTexture = Value
                                UnitBarsF.TargetHealth:SetAttr('bar', 'texture')
                              end,
                      },
                      Width = {
                        type = 'range',
                        name = 'Width',
                        order = 2,
                        min = UnitBarWidthMin,
                        max = UnitBarWidthMax,
                        step = 1,
                      },
                      Height = {
                        type = 'range',
                        name = 'Height',
                        order = 3,
                        min = UnitBarHeightMin,
                        max = UnitBarHeightMax,
                        step = 1,
                      },
                      BarColor = {
                        type = 'color',
                        name = 'Color',
                        order = 4,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.TargetHealth.BarColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.TargetHealth.BarColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.TargetHealth:SetAttr('bar', 'color')
                              end,
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 2,
                    get = function(Info)
                            return UnitBars.TargetHealth.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetHealth.Padding[Info[#Info]] = Value
                            UnitBarsF.TargetHealth:SetAttr('bar', 'padding')
                          end,
                    args = {
                      Left = {
                        type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
            },
          },
--=============================================================================
--        Target Power group.
--=============================================================================
          TargetPower = {
            type = 'group',
            name = 'Target Power',
            order = 4,
            args = {
-------------------------------------------------------------------------------
--            Status - Target Power Group.
-------------------------------------------------------------------------------
              Status = {
                type = 'group',
                name = 'Status',
                dialogInline = true,
                order = 1,
                get = function(Info)
                        return UnitBars.TargetPower.Status[Info[#Info]]
                      end,
                set = function(Info, Value)
                        UnitBars.TargetPower.Status[Info[#Info]] = Value

                        -- Must do a status check/update.
                        GUB:UnitBarsUpdateStatus()
                      end,
                args = {
                  ShowNever = {
                    type = 'toggle',
                    name = 'Never Show',
                    order = 1,
                    desc = 'Disables and hides the bar',
                  },
                  HideWhenDead = {
                    type = 'toggle',
                    name = "Hide when Dead",
                    order = 2,
                    desc = "Hides the bar when you're dead",
                  },
                  HideInVehicle = {
                    type = 'toggle',
                    name = 'Hide when in a Vehicle',
                    order = 3,
                    desc = "Hides the bar when your're in a vehicle",
                  },
                  ShowAlways = {
                    type = 'toggle',
                    name = 'Show Always',
                    order = 4,
                    desc = 'Bar will always be shown',
                  },
                  ShowActive = {
                    type = 'toggle',
                    name = 'Show when Active',
                    order = 5,
                    desc = 'Bar will be shown if there is activty in it',
                  },
                  HideNoCombat = {
                    type = 'toggle',
                    name = 'Hide out of Combat',
                    order = 6,
                    desc = 'When not in combat the bar will be hidden',
                  },
                },
              },
              TextType = {
                type = 'select',
                name = 'Text Type',
                order = 2,
                values = TextTypeDropdown,
                style = 'dropdown',
                desc = 'Changes the look of the value text in the bar',
                get = function()
                        return UnitBars.TargetPower.TextType
                      end,
                set = function(Info, Value)
                        UnitBars.TargetPower.TextType = Value
                           -- Redraw the bar to show the texttype change.
                        UnitBarsF.TargetPower:Update()
                      end,

              },
              BlankLine = {
                type = 'header',
                name = '',
                order = 3,
              },
              Reset = {
                type = 'execute',
                name = 'Reset to Defaults',
                order = 4,
                desc = 'Resets back to the defaults for this bar without changing its location',
                confirm = true,
                func = function()
                         -- Preserve bar location
                         local UB = UnitBars.TargetPower
                         local x, y =  UB.x, UB.y

                         GUB.UnitBars:CopyTableValues(Defaults.profile.TargetPower, UB)

                         UB.x, UB.y = x, y

                         -- Redo everything to show any possible changes.
                         GUB:OnEnable()
                       end,
              },
-------------------------------------------------------------------------------
--            Background - Target Power Group.
-------------------------------------------------------------------------------
              Background = {
                type = 'group',
                name = 'Background',
                order = 10,
                args = {
                  Textures = {
                    type = 'group',
                    name = 'Textures',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.TargetPower.BackdropSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetPower.BackdropSettings[Info[#Info]] = Value
                            UnitBarsF.TargetPower:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      BdTexture = {
                        type = 'select',
                        name = 'Border',
                        order = 1,
                        dialogControl = 'LSM30_Border',
                        values = LSM:HashTable('border'),
                      },
                      BgTexture = {
                        type = 'select',
                        name = 'Background',
                        order = 2,
                        dialogControl = 'LSM30_Background',
                        values = LSM:HashTable('background'),
                      },
                      BdSize = {
                        type = 'range',
                        name = 'Border Thickness',
                        order = 3,
                        min = UnitBarBorderSizeMin,
                        max = UnitBarBorderSizeMax,
                        step = 2,
                      },
                      BgColor = {
                        type = 'color',
                        name = 'Background Color',
                        order = 4,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.TargetPower.BgColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.TargetPower.BgColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.TargetPower:SetAttr('bg', 'color')
                              end,
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 2,
                    get = function(Info)
                            return UnitBars.TargetPower.BackdropSettings.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetPower.BackdropSettings.Padding[Info[#Info]] = Value
                            UnitBarsF.TargetPower:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      Left = {
                      type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Text - Target Power Group.
-------------------------------------------------------------------------------
              Text = {
                type = 'group',
                name = 'Text',
                order = 11,
                args = {
                  Font = {
                    type = 'group',
                    name = 'Font',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.TargetPower.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetPower.FontSettings[Info[#Info]] = Value
                            UnitBarsF.TargetPower:SetAttr('text', 'font')
                          end,
                    args = {
                      FontType = {
                        type = 'select',
                        name = 'Type',
                        order = 1,
                        dialogControl = 'LSM30_Font',
                        values = LSM:HashTable('font'),
                      },
                      FontStyle = {
                        type = 'select',
                        name = 'Style',
                        order = 2,
                        style = 'dropdown',
                        values = FontStyleDropdown,
                      },
                      FontHAlign = {
                        type = 'select',
                        name = 'Horizontal Alignment',
                        order = 3,
                        style = 'dropdown',
                        values = FontHAlignDropDown,
                      },
                      FontSize = {
                        type = 'range',
                        name = 'Size',
                        order = 4,
                        min = UnitBarFontSizeMin,
                        max = UnitBarFontSizeMax,
                        step = 1,
                      },
                      TextColor = {
                        type = 'color',
                        name = 'Color',
                        order = 5,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.TargetPower.TextColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.TargetPower.TextColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.TargetPower:SetAttr('text', 'color')
                              end,
                      },
                    },
                  },
                  Offsets = {
                    type = 'group',
                    name = 'Offsets',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.TargetPower.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetPower.FontSettings[Info[#Info]] = Value
                            UnitBarsF.TargetPower:SetAttr('text', 'font')
                          end,
                    args = {
                      OffsetX = {
                        type = 'range',
                        name = 'Horizonal',
                        order = 4,
                        min = FontOffsetXMin,
                        max = FontOffsetXMax,
                        step = 1,
                      },
                      OffsetY = {
                        type = 'range',
                        name = 'Vertical',
                        order = 5,
                        min = FontOffsetYMin,
                        max = FontOffsetYMax,
                        step = 1,
                      },
                      ShadowOffset = {
                        type = 'range',
                        name = 'Shadow',
                        order = 6,
                        min = FontShadowOffsetMin,
                        max = FontShadowOffsetMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Bar - Target Power Group.
-------------------------------------------------------------------------------
              Bar = {
                type = 'group',
                name = 'Bar',
                order = 12,
                args = {
                  General = {
                    type = 'group',
                    name = 'General',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.TargetPower[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetPower[Info[#Info]] = Value
                            UnitBarsF.TargetPower:SetAttr('bar', 'size')
                          end,
                    args = {
                      StatusBarTexture = {
                        type = 'select',
                        name = 'Bar Texture',
                        order = 1,
                        dialogControl = 'LSM30_Statusbar',
                        values = LSM:HashTable('statusbar'),
                        get = function()
                                return UnitBars.TargetPower.StatusBarTexture
                              end,
                        set = function(Info, Value)
                                UnitBars.TargetPower.StatusBarTexture = Value
                                UnitBarsF.TargetPower:SetAttr('bar', 'texture')
                              end,
                      },
                      Width = {
                        type = 'range',
                        name = 'Width',
                        order = 2,
                        min = UnitBarWidthMin,
                        max = UnitBarWidthMax,
                        step = 1,
                      },
                      Height = {
                        type = 'range',
                        name = 'Height',
                        order = 3,
                        min = UnitBarHeightMin,
                        max = UnitBarHeightMax,
                        step = 1,
                      },
                    },
                  },
                  PowerColors = {
                    type = 'group',
                    name = 'Power Colors',
                    order = 2,
                    dialogInline = true,
                    get = function(Info)
                            -- Info.arg[1] = power color index.
                            local c = UnitBars.TargetPower.BarColor[Info.arg[1]]
                            return c.r, c.g, c.b, c.a
                          end,
                    set = function(Info, r, g, b, a)
                            local c = UnitBars.TargetPower.BarColor[Info.arg[1]]
                            c.r, c.g, c.b, c.a = r, g, b, a

                            -- Update the bar to show the current power color change in real time.
                            UnitBarsF.TargetPower:Update()
                          end,
                    args = {
                      ManaColor = {
                        type = 'color',
                        name = 'Mana',
                        order = 1,
                        hasAlpha = true,
                        arg = {0},
                      },
                      RageColor = {
                        type = 'color',
                        name = 'Rage',
                        order = 2,
                        hasAlpha = true,
                        arg = {1},
                      },
                      FocusColor = {
                        type = 'color',
                        name = 'Focus',
                        order = 3,
                        hasAlpha = true,
                        arg = {2},
                      },
                      EnergyColor = {
                        type = 'color',
                        name = 'Energy',
                        order = 4,
                        hasAlpha = true,
                        arg = {3},
                      },
                      RunicColor = {
                        type = 'color',
                        name = 'Runic Power',
                        order = 5,
                        hasAlpha = true,
                        arg = {6},
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 3,
                    get = function(Info)
                            return UnitBars.TargetPower.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.TargetPower.Padding[Info[#Info]] = Value
                            UnitBarsF.TargetPower:SetAttr('bar', 'padding')
                          end,
                    args = {
                      Left = {
                        type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
            },
          },
--=============================================================================
--        Main Power group.
--=============================================================================
          MainPower = {
            type = 'group',
            name = 'Main Power',
            order = 5,
            args = {
-------------------------------------------------------------------------------
--            Status - Main Power Group.
-------------------------------------------------------------------------------
              Status = {
                type = 'group',
                name = 'Status',
                dialogInline = true,
                order = 1,
                get = function(Info)
                        return UnitBars.MainPower.Status[Info[#Info]]
                      end,
                set = function(Info, Value)
                        UnitBars.MainPower.Status[Info[#Info]] = Value

                        -- Must do a status check/update.
                        GUB:UnitBarsUpdateStatus()
                      end,
                args = {
                  ShowNever = {
                    type = 'toggle',
                    name = 'Never Show',
                    order = 1,
                    desc = 'Disables and hides the bar',
                  },
                  HideWhenDead = {
                    type = 'toggle',
                    name = "Hide when Dead",
                    order = 2,
                    desc = "Hides the bar when you're dead",
                  },
                  HideInVehicle = {
                    type = 'toggle',
                    name = 'Hide when in a Vehicle',
                    order = 3,
                    desc = "Hides the bar when your're in a vehicle",
                  },
                  ShowAlways = {
                    type = 'toggle',
                    name = 'Show Always',
                    order = 4,
                    desc = 'Bar will always be shown',
                  },
                  ShowActive = {
                    type = 'toggle',
                    name = 'Show when Active',
                    order = 5,
                    desc = 'Bar will be shown if there is activty in it',
                  },
                  HideNoCombat = {
                    type = 'toggle',
                    name = 'Hide out of Combat',
                    order = 6,
                    desc = 'When not in combat the bar will be hidden',
                  },
                },
              },
              TextType = {
                type = 'select',
                name = 'Text Type',
                order = 2,
                values = TextTypeDropdown,
                style = 'dropdown',
                desc = 'Changes the look of the value text in the bar',
                get = function()
                        return UnitBars.MainPower.TextType
                      end,
                set = function(Info, Value)
                        UnitBars.MainPower.TextType = Value
                           -- Redraw the bar to show the texttype change.
                        UnitBarsF.MainPower:Update()
                      end,

              },
              BlankLine = {
                type = 'header',
                name = '',
                order = 3,
              },
              Reset = {
                type = 'execute',
                name = 'Reset to Defaults',
                order = 4,
                desc = 'Resets back to the defaults for this bar without changing its location',
                confirm = true,
                func = function()
                         -- Preserve bar location
                         local UB = UnitBars.MainPower
                         local x, y =  UB.x, UB.y

                         GUB.UnitBars:CopyTableValues(Defaults.profile.MainPower, UB)

                         UB.x, UB.y = x, y

                         -- Redo everything to show any possible changes.
                         GUB:OnEnable()
                       end,
              },
-------------------------------------------------------------------------------
--            Background - Main Power Group.
-------------------------------------------------------------------------------
              Background = {
                type = 'group',
                name = 'Background',
                order = 10,
                args = {
                  Textures = {
                    type = 'group',
                    name = 'Textures',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.MainPower.BackdropSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.MainPower.BackdropSettings[Info[#Info]] = Value
                            UnitBarsF.MainPower:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      BdTexture = {
                        type = 'select',
                        name = 'Border',
                        order = 1,
                        dialogControl = 'LSM30_Border',
                        values = LSM:HashTable('border'),
                      },
                      BgTexture = {
                        type = 'select',
                        name = 'Background',
                        order = 2,
                        dialogControl = 'LSM30_Background',
                        values = LSM:HashTable('background'),
                      },
                      BdSize = {
                        type = 'range',
                        name = 'Border Thickness',
                        order = 3,
                        min = UnitBarBorderSizeMin,
                        max = UnitBarBorderSizeMax,
                        step = 2,
                      },
                      BgColor = {
                        type = 'color',
                        name = 'Background Color',
                        order = 4,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.MainPower.BgColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.MainPower.BgColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.MainPower:SetAttr('bg', 'color')
                              end,
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 2,
                    get = function(Info)
                            return UnitBars.MainPower.BackdropSettings.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.MainPower.BackdropSettings.Padding[Info[#Info]] = Value
                            UnitBarsF.MainPower:SetAttr('bg', 'backdrop')
                          end,
                    args = {
                      Left = {
                      type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Text - Main Power Group.
-------------------------------------------------------------------------------
              Text = {
                type = 'group',
                name = 'Text',
                order = 11,
                args = {
                  Font = {
                    type = 'group',
                    name = 'Font',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.MainPower.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.MainPower.FontSettings[Info[#Info]] = Value
                            UnitBarsF.MainPower:SetAttr('text', 'font')
                          end,
                    args = {
                      FontType = {
                        type = 'select',
                        name = 'Type',
                        order = 1,
                        dialogControl = 'LSM30_Font',
                        values = LSM:HashTable('font'),
                      },
                      FontStyle = {
                        type = 'select',
                        name = 'Style',
                        order = 2,
                        style = 'dropdown',
                        values = FontStyleDropdown,
                      },
                      FontHAlign = {
                        type = 'select',
                        name = 'Horizontal Alignment',
                        order = 3,
                        style = 'dropdown',
                        values = FontHAlignDropDown,
                      },
                      FontSize = {
                        type = 'range',
                        name = 'Size',
                        order = 4,
                        min = UnitBarFontSizeMin,
                        max = UnitBarFontSizeMax,
                        step = 1,
                      },
                      TextColor = {
                        type = 'color',
                        name = 'Color',
                        order = 5,
                        hasAlpha = true,
                        get = function()
                                local c = UnitBars.MainPower.TextColor
                                return c.r, c.g, c.b, c.a
                              end,
                        set = function(Info, r, g, b, a)
                                local c = UnitBars.MainPower.TextColor
                                c.r, c.g, c.b, c.a = r, g, b, a
                                UnitBarsF.MainPower:SetAttr('text', 'color')
                              end,
                      },
                    },
                  },
                  Offsets = {
                    type = 'group',
                    name = 'Offsets',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.MainPower.FontSettings[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.MainPower.FontSettings[Info[#Info]] = Value
                            UnitBarsF.MainPower:SetAttr('text', 'font')
                          end,
                    args = {
                      OffsetX = {
                        type = 'range',
                        name = 'Horizonal',
                        order = 4,
                        min = FontOffsetXMin,
                        max = FontOffsetXMax,
                        step = 1,
                      },
                      OffsetY = {
                        type = 'range',
                        name = 'Vertical',
                        order = 5,
                        min = FontOffsetYMin,
                        max = FontOffsetYMax,
                        step = 1,
                      },
                      ShadowOffset = {
                        type = 'range',
                        name = 'Shadow',
                        order = 6,
                        min = FontShadowOffsetMin,
                        max = FontShadowOffsetMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
-------------------------------------------------------------------------------
--            Bar - Main Power Group.
-------------------------------------------------------------------------------
              Bar = {
                type = 'group',
                name = 'Bar',
                order = 12,
                args = {
                  General = {
                    type = 'group',
                    name = 'General',
                    dialogInline = true,
                    order = 1,
                    get = function(Info)
                            return UnitBars.MainPower[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.MainPower[Info[#Info]] = Value
                            UnitBarsF.MainPower:SetAttr('bar', 'size')
                          end,
                    args = {
                      StatusBarTexture = {
                        type = 'select',
                        name = 'Bar Texture',
                        order = 1,
                        dialogControl = 'LSM30_Statusbar',
                        values = LSM:HashTable('statusbar'),
                        get = function()
                                return UnitBars.MainPower.StatusBarTexture
                              end,
                        set = function(Info, Value)
                                UnitBars.MainPower.StatusBarTexture = Value
                                UnitBarsF.MainPower:SetAttr('bar', 'texture')
                              end,
                      },
                      Width = {
                        type = 'range',
                        name = 'Width',
                        order = 2,
                        min = UnitBarWidthMin,
                        max = UnitBarWidthMax,
                        step = 1,
                      },
                      Height = {
                        type = 'range',
                        name = 'Height',
                        order = 3,
                        min = UnitBarHeightMin,
                        max = UnitBarHeightMax,
                        step = 1,
                      },
                    },
                  },
                  PowerColors = {
                    type = 'group',
                    name = 'Power Colors',
                    order = 2,
                    dialogInline = true,
                    get = function(Info)
                            -- Info.arg[1] = power color index.
                            local c = UnitBars.MainPower.BarColor[Info.arg[1]]
                            return c.r, c.g, c.b, c.a
                          end,
                    set = function(Info, r, g, b, a)
                            local c = UnitBars.MainPower.BarColor[Info.arg[1]]
                            c.r, c.g, c.b, c.a = r, g, b, a

                            -- Update the bar to show the current power color change in real time.
                            UnitBarsF.MainPower:Update()
                          end,
                    args = {
                      ManaColor = {
                        type = 'color',
                        name = 'Mana',
                        order = 1,
                        hasAlpha = true,
                        arg = {0},
                      },
                      RageColor = {
                        type = 'color',
                        name = 'Rage',
                        order = 2,
                        hasAlpha = true,
                        arg = {1},
                      },
                      FocusColor = {
                        type = 'color',
                        name = 'Focus',
                        order = 3,
                        hasAlpha = true,
                        arg = {2},
                      },
                      EnergyColor = {
                        type = 'color',
                        name = 'Energy',
                        order = 4,
                        hasAlpha = true,
                        arg = {3},
                      },
                      RunicColor = {
                        type = 'color',
                        name = 'Runic Power',
                        order = 5,
                        hasAlpha = true,
                        arg = {6},
                      },
                    },
                  },
                  Padding = {
                    type = 'group',
                    name = 'Padding',
                    dialogInline = true,
                    order = 3,
                    get = function(Info)
                            return UnitBars.MainPower.Padding[Info[#Info]]
                          end,
                    set = function(Info, Value)
                            UnitBars.MainPower.Padding[Info[#Info]] = Value
                            UnitBarsF.MainPower:SetAttr('bar', 'padding')
                          end,
                    args = {
                      Left = {
                        type = 'range',
                        name = 'Left',
                        order = 1,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Right = {
                        type = 'range',
                        name = 'Right',
                        order = 2,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Top = {
                        type = 'range',
                        name = 'Top',
                        order = 3,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                      Bottom = {
                        type = 'range',
                        name = 'Bottom',
                        order = 4,
                        min = UnitBarPaddingMin,
                        max = UnitBarPaddingMax,
                        step = 1,
                      },
                    },
                  },
                },
              },
            },
          },
---------------------- END OF MAIN POWER
        },
      },
--=============================================================================
-------------------------------------------------------------------------------
--    PROFILES group.
-------------------------------------------------------------------------------
--=============================================================================
      Profile = UnitBarsProfileOptions,
      -- order = 100 -- make sure its always at the end.
    },
  }
  return GUIOptions
end
--=============================================================================
--  Copy Settings group.
--=============================================================================
--[[    CopySettings = {
      type = 'group',
      name = 'Copy Settings',
      order = 3,
      args = {
        PlayerHealth = {
          type = 'group',
          name = 'Player Health',
          order = 1,
          args = {
            Description = {
              type = 'description',
              name = 'Click on the boxes below to copy settings into Player Health.',
              order = 1,
            },
            CopyPlayerPower = {
              type = 'execute',
              name = 'Player Power',
              order = 2,
            },
            Textures = {
              type = 'group',
              name = 'Textures',
              order = 3,
              args = {
                Description = {
                  type = 'description',
                  name = 'Click on the boxes below to copy textures into Player Health.',
                  order = 1,
                },
                CopyTexturePlayerPower = {
                  type = 'execute',
                  name = 'Player Power',
                  order = 2,
                },
              },
            },
          },
        },
      },
    },
  },
} --]]

-------------------------------------------------------------------------------
-- SetUnitBarsTable
--
-- Sets a reference to the UnitBars table
--
-- Usage SetUnitBarsTable(NewTable)
--
-- NewTable    The new table that UnitBars will reference
-------------------------------------------------------------------------------
function GUB.Options:SetUnitBarsTable(NewTable)
  UnitBars = NewTable
end

--*****************************************************************************
--
-- Options Initialization
--
--*****************************************************************************

-------------------------------------------------------------------------------
-- OnInitialize()
--
-- Initializes the options panel and slash options
-------------------------------------------------------------------------------
function GUB.Options:OnInitialize()

  -- Save the unitbars data from the current profile.
  UnitBars = GUB.UnitBarsDB.profile

  -- Create the unitbars options.
  UnitBarsProfileOptions = LibStub('AceDBOptions-3.0'):GetOptionsTable(GUB.UnitBarsDB)

  UnitBarsSlashOptions = CreateUnitBarsSlashOptions()
  UnitBarsOptions = CreateUnitBarsOptions()

  -- Register the slash command options.
  LibStub('AceConfig-3.0'):RegisterOptionsTable(AddonSlashName, UnitBarsSlashOptions, 'gub')

  -- Register the options panel with aceconfig and add it to blizz options.
  LibStub('AceConfig-3.0'):RegisterOptionsTable(AddonOptionsName, UnitBarsOptions)
  UnitBarsOptionsFrame = LibStub('AceConfigDialog-3.0'):AddToBlizOptions(AddonOptionsName, AddonName)

  -- Add the Profiles UI as a subcategory below the main options.
  LibStub('AceConfig-3.0'):RegisterOptionsTable(AddonProfileName, UnitBarsProfileOptions)
--  UnitBarsProfilesFrame = LibStub('AceConfigDialog-3.0'):AddToBlizOptions(AddonProfileName, 'Profiles', AddonName)
end
